# -*- coding: utf-8 -*-
"""
A list of Romanian counties as `choices` in a formfield.

This exists as a standalone file so that it's only imported into memory when
explicitly needed.
"""
from __future__ import unicode_literals

COUNTIES_CHOICES = (
    ('AB', 'Alba'),
    ('AR', 'Arad'),
    ('AG', 'Argeş'),
    ('BC', 'Bacău'),
    ('BH', 'Bihor'),
    ('BN', 'Bistriţa-Năsăud'),
    ('BT', 'Botoşani'),
    ('BV', 'Braşov'),
    ('BR', 'Brăila'),
    ('B',  'Bucureşti'),
    ('BZ', 'Buzău'),
    ('CS', 'Caraş-Severin'),
    ('CL', 'Călăraşi'),
    ('CJ', 'Cluj'),
    ('CT', 'Constanţa'),
    ('CV', 'Covasna'),
    ('DB', 'Dâmboviţa'),
    ('DJ', 'Dolj'),
    ('GL', 'Galaţi'),
    ('GR', 'Giurgiu'),
    ('GJ', 'Gorj'),
    ('HR', 'Harghita'),
    ('HD', 'Hunedoara'),
    ('IL', 'Ialomiţa'),
    ('IS', 'Iaşi'),
    ('IF', 'Ilfov'),
    ('MM', 'Maramureş'),
    ('MH', 'Mehedinţi'),
    ('MS', 'Mureş'),
    ('NT', 'Neamţ'),
    ('OT', 'Olt'),
    ('PH', 'Prahova'),
    ('SM', 'Satu Mare'),
    ('SJ', 'Sălaj'),
    ('SB', 'Sibiu'),
    ('SV', 'Suceava'),
    ('TR', 'Teleorman'),
    ('TM', 'Timiş'),
    ('TL', 'Tulcea'),
    ('VS', 'Vaslui'),
    ('VL', 'Vâlcea'),
    ('VN', 'Vrancea'),
)
