{% extends "admin/base_site.html" %}
{% load i18n %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; {% trans 'Documentation' %}</a>
</div>
{% endblock %}
{% block title %}Please install docutils{% endblock %}

{% block content %}

<h1>Documentation</h1>

<div id="content-main">
  <h3>The admin documentation system requires Python's <a href="http://docutils.sf.net/">docutils</a> library.</h3>

  <p>Please ask your administrators to install <a href="http://docutils.sf.net/">docutils</a>.</p>
</div>

{% endblock %}
