from __future__ import absolute_import

from .debug import (DebugViewTests, ExceptionReporterTests,
    ExceptionReporterTests, PlainTextReportTests, ExceptionReporterFilterTests,
    AjaxResponseExceptionReporterFilter)
from .defaults import DefaultsTests
from .i18n import JsI18NTests, I18NTests, JsI18NTestsMultiPackage, JavascriptI18nTests
from .shortcuts import ShortcutTests
from .specials import URLHandling
from .static import StaticHelperTest, StaticUtilsTests, StaticTests
