{% extends "admin/base_site.html" %}
{% load i18n %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; {% trans 'Documentation' %}</a>
</div>
{% endblock %}
{% block title %}Documentation{% endblock %}

{% block content %}

<h1>Documentation</h1>

<div id="content-main">
  <h3><a href="tags/">Tags</a></h3>
  <p>List of all the template tags and their functions.</p>

  <h3><a href="filters/">Filters</a></h3>
  <p>Filters are actions which can be applied to variables in a template to alter the output.</p>

  <h3><a href="models/">Models</a></h3>
  <p>Models are descriptions of all the objects in the system and their associated fields. Each model has a list of fields which can be accessed as template variables.</p>

  <h3><a href="views/">Views</a></h3>
  <p>Each page on the public site is generated by a view. The view defines which template is used to generate the page and which objects are available to that template.</p>

    <h3><a href="bookmarklets/">Bookmarklets</a></h3>
    <p>Tools for your browser to quickly access admin functionality.</p>
</div>

{% endblock %}

