# -*- coding: utf-8 -*-
"""
Macedonian municipalities per the reorganization from 2004.
"""
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

MK_MUNICIPALITIES = (
    ('AD', _('Aerodrom')),
    ('AR', _('Aračinovo')),
    ('BR', _('Berovo')),
    ('TL', _('Bitola')),
    ('BG', _('Bogdanci')),
    ('VJ', _('Bogovinje')),
    ('BS', _('Bosilovo')),
    ('BN', _('Brvenica')),
    ('BU', _('Butel')),
    ('VA', _('Valandovo')),
    ('VL', _('Vasilevo')),
    ('VV', _('Vevčani')),
    ('VE', _('Veles')),
    ('NI', _('Vinica')),
    ('VC', _('Vraneštica')),
    ('VH', _('Vrapčište')),
    ('GB', _('Gazi Baba')),
    ('GV', _('Gevgelija')),
    ('GT', _('Gostivar')),
    ('GR', _('Gradsko')),
    ('DB', _('Debar')),
    ('DA', _('Debarca')),
    ('DL', _('Delčevo')),
    ('DK', _('Demir Kapija')),
    ('DM', _('Demir Hisar')),
    ('DE', _('Dolneni')),
    ('DR', _('Drugovo')),
    ('GP', _('Gjorče Petrov')),
    ('ZE', _('Želino')),
    ('ZA', _('Zajas')),
    ('ZK', _('Zelenikovo')),
    ('ZR', _('Zrnovci')),
    ('IL', _('Ilinden')),
    ('JG', _('Jegunovce')),
    ('AV', _('Kavadarci')),
    ('KB', _('Karbinci')),
    ('KX', _('Karpoš')),
    ('VD', _('Kisela Voda')),
    ('KH', _('Kičevo')),
    ('KN', _('Konče')),
    ('OC', _('Koćani')),
    ('KY', _('Kratovo')),
    ('KZ', _('Kriva Palanka')),
    ('KG', _('Krivogaštani')),
    ('KS', _('Kruševo')),
    ('UM', _('Kumanovo')),
    ('LI', _('Lipkovo')),
    ('LO', _('Lozovo')),
    ('MR', _('Mavrovo i Rostuša')),
    ('MK', _('Makedonska Kamenica')),
    ('MD', _('Makedonski Brod')),
    ('MG', _('Mogila')),
    ('NG', _('Negotino')),
    ('NV', _('Novaci')),
    ('NS', _('Novo Selo')),
    ('OS', _('Oslomej')),
    ('OD', _('Ohrid')),
    ('PE', _('Petrovec')),
    ('PH', _('Pehčevo')),
    ('PN', _('Plasnica')),
    ('PP', _('Prilep')),
    ('PT', _('Probištip')),
    ('RV', _('Radoviš')),
    ('RN', _('Rankovce')),
    ('RE', _('Resen')),
    ('RO', _('Rosoman')),
    ('AJ', _('Saraj')),
    ('SL', _('Sveti Nikole')),
    ('SS', _('Sopište')),
    ('SD', _('Star Dojran')),
    ('NA', _('Staro Nagoričane')),
    ('UG', _('Struga')),
    ('RU', _('Strumica')),
    ('SU', _('Studeničani')),
    ('TR', _('Tearce')),
    ('ET', _('Tetovo')),
    ('CE', _('Centar')),
    ('CZ', _('Centar-Župa')),
    ('CI', _('Čair')),
    ('CA', _('Čaška')),
    ('CH', _('Češinovo-Obleševo')),
    ('CS', _('Čučer-Sandevo')),
    ('ST', _('Štip')),
    ('SO', _('Šuto Orizari')),
)
