from django.utils.translation import ugettext_lazy

JP_PREFECTURES = (
    ('hokkaido', ugettext_lazy('Hokkaido'),),
    ('aomori', ugettext_lazy('Aomori'),),
    ('iwate', ugettext_lazy('Iwate'),),
    ('miyagi', ugettext_lazy('Miyagi'),),
    ('akita', ugettext_lazy('Akita'),),
    ('yamagata', ugettext_lazy('Yamagata'),),
    ('fukushima', ugettext_lazy('Fukushima'),),
    ('ibaraki', ugettext_lazy('Ibaraki'),),
    ('tochigi', ugettext_lazy('Tochigi'),),
    ('gunma', ugettext_lazy('Gunma'),),
    ('saitama', ugettext_lazy('Saitama'),),
    ('chiba', ugettext_lazy('Chiba'),),
    ('tokyo', ugettext_lazy('Tokyo'),),
    ('kanagawa', ugettext_lazy('Kanagawa'),),
    ('yamanashi', ugettext_lazy('Yamanashi'),),
    ('nagano', ugettext_lazy('Nagano'),),
    ('niigata', ugettext_lazy('Niigata'),),
    ('toyama', ugettext_lazy('Toyama'),),
    ('ishikawa', ugettext_lazy('Ishikawa'),),
    ('fukui', ugettext_lazy('Fukui'),),
    ('gifu', ugettext_lazy('Gifu'),),
    ('shizuoka', ugettext_lazy('Shizuoka'),),
    ('aichi', ugettext_lazy('Aichi'),),
    ('mie', ugettext_lazy('Mie'),),
    ('shiga', ugettext_lazy('Shiga'),),
    ('kyoto', ugettext_lazy('Kyoto'),),
    ('osaka', ugettext_lazy('Osaka'),),
    ('hyogo', ugettext_lazy('Hyogo'),),
    ('nara', ugettext_lazy('Nara'),),
    ('wakayama', ugettext_lazy('Wakayama'),),
    ('tottori', ugettext_lazy('Tottori'),),
    ('shimane', ugettext_lazy('Shimane'),),
    ('okayama', ugettext_lazy('Okayama'),),
    ('hiroshima', ugettext_lazy('Hiroshima'),),
    ('yamaguchi', ugettext_lazy('Yamaguchi'),),
    ('tokushima', ugettext_lazy('Tokushima'),),
    ('kagawa', ugettext_lazy('Kagawa'),),
    ('ehime', ugettext_lazy('Ehime'),),
    ('kochi', ugettext_lazy('Kochi'),),
    ('fukuoka', ugettext_lazy('Fukuoka'),),
    ('saga', ugettext_lazy('Saga'),),
    ('nagasaki', ugettext_lazy('Nagasaki'),),
    ('kumamoto', ugettext_lazy('Kumamoto'),),
    ('oita', ugettext_lazy('Oita'),),
    ('miyazaki', ugettext_lazy('Miyazaki'),),
    ('kagoshima', ugettext_lazy('Kagoshima'),),
    ('okinawa', ugettext_lazy('Okinawa'),),
)
