# -*- coding: utf-8 -*-

# http://www.statoids.com/upy.html
from __future__ import unicode_literals

DEPARTMENT_CHOICES = (
    ('AG', 'Alto Paraguay'),
    ('AA', 'Alto Paraná'),
    ('AM', 'Amambay'),
    ('AS', 'Asunción'),
    ('BQ', 'Boquerón'),
    ('CG', 'Caaguazú'),
    ('CZ', 'Caazapá'),
    ('CY', 'Canindeyú'),
    ('CE', 'Central'),
    ('CN', 'Concepción'),
    ('CR', 'Cordillera'),
    ('GU', 'Guairá'),
    ('IT', 'Itapúa'),
    ('MI', 'Misiones'),
    ('NE', 'Ñeembucú'),
    ('PG', 'Paraguarí'),
    ('PH', 'Pdte. Hayes'),
    ('SP', 'San Pedro'),
)

DEPARTMENT_ROMAN_CHOICES = (
    ('CN', 'I Concepción'),
    ('SP', 'II San Pedro'),
    ('CR', 'III Cordillera'),
    ('GU', 'IV Guairá'),
    ('CG', 'V Caaguazú'),
    ('CZ', 'VI Caazapá'),
    ('IT', 'VII Itapúa'),
    ('MI', 'VIII Misiones'),
    ('PG', 'IX Paraguarí'),
    ('AA', 'X Alto Paraná'),
    ('CE', 'XI Central'),
    ('NE', 'XII Ñeembucú'),
    ('AM', 'XIII Amambay'),
    ('CY', 'XIV Canindeyú'),
    ('PH', 'XV Pdte. Hayes'),
    ('AG', 'XVI Alto Paraguay'),
    ('BQ', 'XVII Boquerón'),
    ('AS', 'XVIII Asunción'),
)
