{% extends "comments/base.html" %}
{% load i18n %}

{% block title %}{% trans "Flag this comment" %}{% endblock %}

{% block content %}
<h1>{% trans "Really flag this comment?" %}</h1>
  <blockquote>{{ comment|linebreaks }}</blockquote>
  <form action="." method="post">{% csrf_token %}
    {% if next %}<div><input type="hidden" name="next" value="{{ next }}" id="next" /></div>{% endif %}
    <p class="submit">
    <input type="submit" name="submit" value="{% trans "Flag" %}" /> or <a href="{{ comment.get_absolute_url }}">cancel</a>
    </p>
  </form>
{% endblock %}
