# -*- coding: utf-8 -*-
"""A list of Urguayan departaments as `choices` in a formfield."""
from __future__ import unicode_literals

DEPARTAMENT_CHOICES = (
    ('G', 'Artigas'),
    ('A', 'Canelones'),
    ('E', 'Cerro Largo'),
    ('L', 'Colonia'),
    ('Q', 'Durazno'),
    ('N', 'Flores'),
    ('O', 'Florida'),
    ('P', 'Lavalleja'),
    ('B', 'Maldonado'),
    ('S', 'Montevideo'),
    ('I', 'Paysandú'),
    ('J', 'Río Negro'),
    ('F', 'Rivera'),
    ('C', 'Rocha'),
    ('H', 'Salto'),
    ('M', 'San José'),
    ('K', 'Soriano'),
    ('R', 'Tacuarembó'),
    ('D', 'Treinta y Tres'),
)
